from exceptions import LookupError
##
##Ajgu, a graphe database in Python for Python users
##Copyright (C) 2010 Amirouche Boubekki <amirouche.boubekki@gmail.com>
##
##This program is free software: you can redistribute it and/or modify
##it under the terms of the GNU Affero General Public License as published by
##the Free Software Foundation, either version 3 of the License, or
##(at your option) any later version.
##
##This program is distributed in the hope that it will be useful,
##but WITHOUT ANY WARRANTY; without even the implied warranty of
##MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##GNU Affero General Public License for more details.
##
##You should have received a copy of the GNU Affero General Public License
##along with this program.  If not, see <http://www.gnu.org/licenses/>.
##
import anydbm as db

from persistence_backend import PersistenceBackend

class AnyDbBackend(PersistenceBackend):
    def __init__(self, options):
        try:
            db_home = options["db_home"]
        except:
            raise LookupError("`db_home` option is mandatory to run AnyDbBackend")
        try :
            flag = options["flag"]
        except:
            flag = "n"
        try :
            mode = options["mode"]
        except:
            mode = 0655
        self.db = db.open(db_home, flag, mode)


    def get_property(self, class_name, id, type, key):
        db_key = AnyDbBackend.get_property_code_for_key(class_name, id, type, key)
        return self.db[db_key]

    def set_property(self, class_name, id, type, key, property):
        db_key = AnyDbBackend.get_property_code_for_key(class_name, id, type, key)
        self.db[db_key] = property


    def keys(self, class_name, id):
        keys_ = list()
        db_key = AnyDbBackend.get_object_properties_code(class_name, id)
        for key in self.db.keys():
            if key.startswith(db_key):
                keys_.append(key.split(":")[-1])
        return keys_


    def get_class_count(self, class_name):
        db_key = AnyDbBackend.get_class_property_code(class_name, "count")        
        if self.db.has_key(db_key):
            courant = long(self.db[db_key])
            return courant
        else:
            return 0

    def get_next_id(self, class_name):
        db_key = AnyDbBackend.get_class_property_code(class_name, "count")
        if self.db.has_key(db_key):
            courant = long(self.db[db_key])
            self.db[db_key] = str(courant + 1)
            return courant
        else:
            self.db[db_key] = str(1)
            return 0

    def delete(self, class_name, id):
        keys_ = list()
        db_key = AnyDbBackend.get_object_code(class_name, id)
        for key in self.db.keys():
            if key.startswith(db_key):
                del self.db[key]
        return keys_


    def _print(self):
        for key in self.db.keys():
            print("%s -> %s" % (key, self.db[key]))
