from abc import ABCMeta, abstractmethod
##
##Ajgu, a graphe database in Python for Python users
##Copyright (C) 2010 Amirouche Boubekki <amirouche.boubekki@gmail.com>
##
##This program is free software: you can redistribute it and/or modify
##it under the terms of the GNU Affero General Public License as published by
##the Free Software Foundation, either version 3 of the License, or
##(at your option) any later version.
##
##This program is distributed in the hope that it will be useful,
##but WITHOUT ANY WARRANTY; without even the implied warranty of
##MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##GNU Affero General Public License for more details.
##
##You should have received a copy of the GNU Affero General Public License
##along with this program.  If not, see <http://www.gnu.org/licenses/>.
##

class PersistenceBackend:
    __metaclass__ = ABCMeta
    
    @staticmethod
    def get_object_code(class_name, id):
        return "%s:%d" % (class_name, id)

    @staticmethod
    def get_object_properties_code(class_name, id):
        return "%s:%d:key" % (class_name, id)

    @staticmethod
    def get_property_code_for_key(class_name, id, type, key):
        return "%s:%d:%s:%s" % (class_name, id, type, key)

    @staticmethod
    def get_class_property_code(class_name, property):
        return "%s:%s" % (class_name, property)


    @abstractmethod
    def get_property(self, class_name, id, type, key):
        pass

    @abstractmethod
    def set_property(self, class_name, id, type, key, property):
        pass


    @abstractmethod
    def keys(self, class_name, id, type):
        pass


    @abstractmethod
    def get_class_count(self, class_name):
        pass

    @abstractmethod
    def get_next_id(self, class_name):
        pass


    @abstractmethod    
    def delete(self, class_name, id):
        pass
