import hashlib
##
##Ajgu, a graphe database in Python for Python users
##Copyright (C) 2010 Amirouche Boubekki <amirouche.boubekki@gmail.com>
##
##This program is free software: you can redistribute it and/or modify
##it under the terms of the GNU Affero General Public License as published by
##the Free Software Foundation, either version 3 of the License, or
##(at your option) any later version.
##
##This program is distributed in the hope that it will be useful,
##but WITHOUT ANY WARRANTY; without even the implied warranty of
##MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##GNU Affero General Public License for more details.
##
##You should have received a copy of the GNU Affero General Public License
##along with this program.  If not, see <http://www.gnu.org/licenses/>.
##
from exceptions import LookupError, TypeError

from backends.utils import validate_key
from backends.utils import validate_property
from backends.utils import validate_private_key

class PersistentDictionnary:
    def __init__(self, backend, id=None):
        self._backend = backend

        if id != None:
            self._id = id
        else:
            self._id = self._backend.get_next_id(self.__class__.__name__)
            self._set_private_property("", "")
    
    def _set_private_property(self, key, property):
        validate_private_key(key)
        validate_property(property)

        self._backend.set_property(self.__class__.__name__, 
                                  self._id,
                                  "private",
                                  key,
                                  property)

    def _get_private_property(self, key):
        validate_private_key(key)
        return self._backend.get_property(self.__class__.__name__,
                                         self._id,
                                         "private",
                                         key)

    def get_property(self, key):
        validate_key(key)
        try:
            return self._backend.get_property(self.__class__.__name__,
                                             self._id,
                                             "key",
                                             key)
        except:
            raise KeyError(key)

    def set_property(self, key, property):
        validate_key(key)
        validate_property(property)

        self._backend.set_property(self.__class__.__name__,
                                  self._id,
                                  "key",
                                  key,
                                  property)
    
    def _keys(self):
        return self._backend.keys(self.__class__.__name__,
                                    self._id)
    
    def delete(self):
        self._backend.delete(self.__class__.__name__,
                            self._id)

    def __hash__(self):
        return hashlib.new("%s%s" % (self.__class.__name__, str(self._id)))
        
    def __repr__(self):
        return "<%s id:%s>" % (self.__class__.__name__, self._id)
    
    def __eq__(self, other):
        if self.__class__.__name__ != other.__class__.__name__:
            return False
        return self._id == other._id 

    def __cmp__(self, other):
        if self == other:
            return 0
        else :
            return 1
